@@ -13,11 +13,14 @@
 #include "mozilla/dom/GeometryUtilsBinding.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/Text.h"
+#include "mozilla/dom/DocumentInlines.h"
 #include "mozilla/dom/DOMPoint.h"
 #include "mozilla/dom/DOMQuad.h"
 #include "mozilla/dom/DOMRect.h"
 #include "mozilla/dom/BrowserChild.h"
 #include "nsIFrame.h"
+#include "nsContainerFrame.h"
+#include "nsContentUtils.h"
 #include "nsCSSFrameConstructor.h"
 #include "nsLayoutUtils.h"
 
@@ -191,7 +194,7 @@ class AccumulateQuadCallback : public nsLayoutUtils::BoxCallback {
     }
   }
 
-  virtual void AddBox(nsIFrame* aFrame) override {
+  void AddBox(nsIFrame* aFrame) override {
     nsIFrame* f = aFrame;
     if (mBoxType == CSSBoxType::Margin && f->IsTableFrame()) {
       // Margin boxes for table frames should be taken from the table wrapper
@@ -207,8 +210,8 @@ class AccumulateQuadCallback : public nsLayoutUtils::BoxCallback {
           CSSPoint(nsPresContext::AppUnitsToFloatCSSPixels(appUnits[i].x),
                    nsPresContext::AppUnitsToFloatCSSPixels(appUnits[i].y));
     }
-    nsLayoutUtils::TransformResult rv =
-        nsLayoutUtils::TransformPoints(f, mRelativeToFrame, 4, points);
+    nsLayoutUtils::TransformResult rv = nsLayoutUtils::TransformPoints(
+        RelativeTo{f}, RelativeTo{mRelativeToFrame}, 4, points);
     if (rv == nsLayoutUtils::TRANSFORM_SUCCEEDED) {
       CSSPoint delta(
           nsPresContext::AppUnitsToFloatCSSPixels(mRelativeToBoxTopLeft.x),
@@ -422,8 +425,8 @@ static void TransformPoints(nsINode* aTo, const GeometryNode& aFrom,
   for (uint32_t i = 0; i < aPointCount; ++i) {
     aPoints[i] += fromOffsetGfx;
   }
-  nsLayoutUtils::TransformResult rv =
-      nsLayoutUtils::TransformPoints(fromFrame, toFrame, aPointCount, aPoints);
+  nsLayoutUtils::TransformResult rv = nsLayoutUtils::TransformPoints(
+      RelativeTo{fromFrame}, RelativeTo{toFrame}, aPointCount, aPoints);
   if (rv == nsLayoutUtils::TRANSFORM_SUCCEEDED) {
     CSSPoint toOffsetGfx(nsPresContext::AppUnitsToFloatCSSPixels(toOffset.x),
                          nsPresContext::AppUnitsToFloatCSSPixels(toOffset.y));
