@@ -26,17 +26,16 @@ class nsFontMetrics;
 class nsImageMap;
 class nsIURI;
 class nsILoadGroup;
-class nsDisplayImage;
 class nsPresContext;
 class nsImageFrame;
 class nsTransform2D;
 class nsImageLoadingContent;
 
 namespace mozilla {
+class nsDisplayImage;
 class PresShell;
 namespace layers {
 class ImageContainer;
-class ImageLayer;
 class LayerManager;
 }  // namespace layers
 }  // namespace mozilla
@@ -66,13 +65,12 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
 
   typedef mozilla::image::ImgDrawResult ImgDrawResult;
   typedef mozilla::layers::ImageContainer ImageContainer;
-  typedef mozilla::layers::ImageLayer ImageLayer;
   typedef mozilla::layers::LayerManager LayerManager;
 
   NS_DECL_FRAMEARENA_HELPERS(nsImageFrame)
   NS_DECL_QUERYFRAME
 
-  void DestroyFrom(nsIFrame* aDestructRoot, PostDestroyData&) override;
+  void Destroy(DestroyContext&) override;
   void DidSetComputedStyle(ComputedStyle* aOldStyle) final;
 
   void Init(nsIContent* aContent, nsContainerFrame* aParent,
@@ -86,8 +84,9 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   }
   void Reflow(nsPresContext*, ReflowOutput&, const ReflowInput&,
               nsReflowStatus&) override;
+  bool IsLeafDynamic() const override;
 
-  nsresult GetContentForEvent(mozilla::WidgetEvent*,
+  nsresult GetContentForEvent(const mozilla::WidgetEvent*,
                               nsIContent** aContent) final;
   nsresult HandleEvent(nsPresContext*, mozilla::WidgetGUIEvent*,
                        nsEventStatus*) override;
@@ -100,9 +99,16 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
       const Maybe<OnNonvisible>& aNonvisibleAction = Nothing()) final;
 
   void ResponsiveContentDensityChanged();
-  void SetupForContentURLRequest();
+  void ElementStateChanged(mozilla::dom::ElementState) override;
+  void SetupOwnedRequest();
+  void DeinitOwnedRequest();
   bool ShouldShowBrokenImageIcon() const;
 
+  bool IsForImageLoadingContent() const {
+    return mKind == Kind::ImageLoadingContent;
+  }
+
+  void UpdateXULImage();
   const mozilla::StyleImage* GetImageFromStyle() const;
 
 #ifdef ACCESSIBILITY
@@ -122,26 +128,30 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
 
   LogicalSides GetLogicalSkipSides() const final;
 
-  static void ReleaseGlobals() {
-    if (gIconLoad) {
-      gIconLoad->Shutdown();
-      gIconLoad = nullptr;
-    }
-  }
-
-  nsresult RestartAnimation();
-  nsresult StopAnimation();
+  static void ReleaseGlobals();
 
   already_AddRefed<imgIRequest> GetCurrentRequest() const;
   void Notify(imgIRequest*, int32_t aType, const nsIntRect* aData);
 
   /**
-   * Function to test whether given an element and its style, that element
-   * should get an image frame.  Note that this method is only used by the
-   * frame constructor; it's only here because it uses gIconLoad for now.
+   * Returns whether we should replace an element with an image corresponding to
+   * its 'content' CSS property.
    */
-  static bool ShouldCreateImageFrameFor(const mozilla::dom::Element&,
-                                        ComputedStyle&);
+  static bool ShouldCreateImageFrameForContentProperty(
+      const mozilla::dom::Element&, const ComputedStyle&);
+
+  /**
+   * Function to test whether given an element and its style, that element
+   * should get an image frame, and if so, which kind of image frame (for
+   * `content`, or for the element itself).
+   */
+  enum class ImageFrameType {
+    ForContentProperty,
+    ForElementRequest,
+    None,
+  };
+  static ImageFrameType ImageFrameTypeFor(const mozilla::dom::Element&,
+                                          const ComputedStyle&);
 
   ImgDrawResult DisplayAltFeedback(gfxContext& aRenderingContext,
                                    const nsRect& aDirtyRect, nsPoint aPt,
@@ -154,8 +164,6 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
       mozilla::layers::RenderRootStateManager*, nsDisplayListBuilder*,
       nsPoint aPt, uint32_t aFlags);
 
-  nsRect GetInnerArea() const;
-
   /**
    * Return a map element associated with this image.
    */
@@ -181,7 +189,9 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   // The kind of image frame we are.
   enum class Kind : uint8_t {
     // For an nsImageLoadingContent.
-    ImageElement,
+    ImageLoadingContent,
+    // For a <xul:image> element.
+    XULImage,
     // For css 'content: url(..)' on non-generated content.
     ContentProperty,
     // For a child of a ::before / ::after pseudo-element that had an url() item
@@ -197,6 +207,7 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
 
  private:
   friend nsIFrame* NS_NewImageFrame(mozilla::PresShell*, ComputedStyle*);
+  friend nsIFrame* NS_NewXULImageFrame(mozilla::PresShell*, ComputedStyle*);
   friend nsIFrame* NS_NewImageFrameForContentProperty(mozilla::PresShell*,
                                                       ComputedStyle*);
   friend nsIFrame* NS_NewImageFrameForGeneratedContentIndex(mozilla::PresShell*,
@@ -209,9 +220,14 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
 
   nsImageFrame(ComputedStyle*, nsPresContext* aPresContext, ClassID, Kind);
 
+  void ReflowChildren(nsPresContext*, const ReflowInput&,
+                      const mozilla::LogicalSize& aImageSize);
+
+  void UpdateIntrinsicSizeAndRatio();
+
  protected:
   nsImageFrame(ComputedStyle* aStyle, nsPresContext* aPresContext, ClassID aID)
-      : nsImageFrame(aStyle, aPresContext, aID, Kind::ImageElement) {}
+      : nsImageFrame(aStyle, aPresContext, aID, Kind::ImageLoadingContent) {}
 
   ~nsImageFrame() override;
 
@@ -231,7 +247,10 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
 
   bool IsServerImageMap();
 
-  void TranslateEventCoords(const nsPoint& aPoint, nsIntPoint& aResult);
+  // Translate a point that is relative to our frame into a localized CSS pixel
+  // coordinate that is relative to the content area of this frame (inside the
+  // border+padding).
+  mozilla::CSSIntPoint TranslateEventCoords(const nsPoint& aPoint);
 
   bool GetAnchorHREFTargetAndNode(nsIURI** aHref, nsString& aTarget,
                                   nsIContent** aNode);
@@ -286,6 +305,13 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   /// Always sync decode our image when painting if @aForce is true.
   void SetForceSyncDecoding(bool aForce) { mForceSyncDecoding = aForce; }
 
+  void AssertSyncDecodingHintIsInSync() const
+#ifndef DEBUG
+      {}
+#else
+      ;
+#endif
+
   /**
    * Computes the predicted dest rect that we'll draw into, in app units, based
    * upon the provided frame content box. (The content box is what
@@ -295,15 +321,7 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   nsRect PredictedDestRect(const nsRect& aFrameContentBox);
 
  private:
-  void MaybeRecordContentUrlOnImageTelemetry();
-
-  // random helpers
-  inline void SpecToURI(const nsAString& aSpec, nsIURI** aURI);
-
-  inline void GetLoadGroup(nsPresContext* aPresContext,
-                           nsILoadGroup** aLoadGroup);
   nscoord GetContinuationOffset() const;
-  void GetDocumentCharacterSet(nsACString& aCharset) const;
   bool ShouldDisplaySelection();
 
   // Whether the image frame should use the mapped aspect ratio from width=""
@@ -335,8 +353,8 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   bool IsPendingLoad(imgIRequest*) const;
 
   /**
-   * Updates mImage based on the current image request (cannot be null), and the
-   * image passed in (can be null), and invalidate layout and paint as needed.
+   * Updates mImage based on the current image request, and the image passed in
+   * (both can be null), and invalidate layout and paint as needed.
    */
   void UpdateImage(imgIRequest*, imgIContainer*);
 
@@ -358,16 +376,25 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   void InvalidateSelf(const nsIntRect* aLayerInvalidRect,
                       const nsRect* aFrameInvalidRect);
 
+  void MaybeSendIntrinsicSizeAndRatioToEmbedder();
+  void MaybeSendIntrinsicSizeAndRatioToEmbedder(Maybe<mozilla::IntrinsicSize>,
+                                                Maybe<mozilla::AspectRatio>);
+
   RefPtr<nsImageMap> mImageMap;
 
   RefPtr<nsImageListener> mListener;
 
-  // An image request created for content: url(..) or list-style-image.
-  RefPtr<imgRequestProxy> mContentURLRequest;
+  // An image request created for content: url(..), list-style-image, or
+  // <xul:image>.
+  RefPtr<imgRequestProxy> mOwnedRequest;
 
   nsCOMPtr<imgIContainer> mImage;
   nsCOMPtr<imgIContainer> mPrevImage;
+
+  // The content-box size as if we are not fragmented, cached in the most recent
+  // reflow.
   nsSize mComputedSize;
+
   mozilla::IntrinsicSize mIntrinsicSize;
 
   // Stores mImage's intrinsic ratio, or a default AspectRatio if there's no
@@ -375,113 +402,49 @@ class nsImageFrame : public nsAtomicContainerFrame, public nsIReflowCallback {
   mozilla::AspectRatio mIntrinsicRatio;
 
   const Kind mKind;
-  bool mContentURLRequestRegistered;
-  bool mDisplayingIcon;
-  bool mFirstFrameComplete;
-  bool mReflowCallbackPosted;
-  bool mForceSyncDecoding;
-
-  /* loading / broken image icon support */
-
-  // XXXbz this should be handled by the prescontext, I think; that
-  // way we would have a single iconload per mozilla session instead
-  // of one per document...
-
-  // LoadIcons: initiate the loading of the static icons used to show
-  // loading / broken images
-  nsresult LoadIcons(nsPresContext* aPresContext);
-  nsresult LoadIcon(const nsAString& aSpec, nsPresContext* aPresContext,
-                    imgRequestProxy** aRequest);
-
-  class IconLoad final : public nsIObserver, public imgINotificationObserver {
-    // private class that wraps the data and logic needed for
-    // broken image and loading image icons
-   public:
-    IconLoad();
-
-    void Shutdown();
-
-    NS_DECL_ISUPPORTS
-    NS_DECL_NSIOBSERVER
-    NS_DECL_IMGINOTIFICATIONOBSERVER
-
-    void AddIconObserver(nsImageFrame* frame) {
-      MOZ_ASSERT(!mIconObservers.Contains(frame),
-                 "Observer shouldn't aleady be in array");
-      mIconObservers.AppendElement(frame);
-    }
-
-    void RemoveIconObserver(nsImageFrame* frame) {
-      mozilla::DebugOnly<bool> didRemove = mIconObservers.RemoveElement(frame);
-      MOZ_ASSERT(didRemove, "Observer not in array");
-    }
-
-   private:
-    ~IconLoad() = default;
-
-    void GetPrefs();
-    nsTObserverArray<nsImageFrame*> mIconObservers;
-
-   public:
-    RefPtr<imgRequestProxy> mLoadingImage;
-    RefPtr<imgRequestProxy> mBrokenImage;
-    bool mPrefForceInlineAltText;
-    bool mPrefShowPlaceholders;
-    bool mPrefShowLoadingPlaceholder;
-  };
+  bool mOwnedRequestRegistered = false;
+  bool mDisplayingIcon = false;
+  bool mFirstFrameComplete = false;
+  bool mReflowCallbackPosted = false;
+  bool mForceSyncDecoding = false;
+  bool mIsInObjectOrEmbed = false;
 
  public:
-  // singleton pattern: one LoadIcons instance is used
-  static mozilla::StaticRefPtr<IconLoad> gIconLoad;
-
-  friend class nsDisplayImage;
+  friend class mozilla::nsDisplayImage;
   friend class nsDisplayGradient;
 };
 
+namespace mozilla {
 /**
  * Note that nsDisplayImage does not receive events. However, an image element
  * is replaced content so its background will be z-adjacent to the
  * image itself, and hence receive events just as if the image itself
  * received events.
  */
-class nsDisplayImage final : public nsDisplayImageContainer {
+class nsDisplayImage final : public nsPaintedDisplayItem {
  public:
   typedef mozilla::layers::LayerManager LayerManager;
 
   nsDisplayImage(nsDisplayListBuilder* aBuilder, nsImageFrame* aFrame,
                  imgIContainer* aImage, imgIContainer* aPrevImage)
-      : nsDisplayImageContainer(aBuilder, aFrame),
+      : nsPaintedDisplayItem(aBuilder, aFrame),
         mImage(aImage),
         mPrevImage(aPrevImage) {
     MOZ_COUNT_CTOR(nsDisplayImage);
   }
   ~nsDisplayImage() final { MOZ_COUNT_DTOR(nsDisplayImage); }
 
-  nsDisplayItemGeometry* AllocateGeometry(nsDisplayListBuilder*) final;
-  void ComputeInvalidationRegion(nsDisplayListBuilder*,
-                                 const nsDisplayItemGeometry*,
-                                 nsRegion* aInvalidRegion) const final;
   void Paint(nsDisplayListBuilder*, gfxContext* aCtx) final;
 
-  already_AddRefed<imgIContainer> GetImage() final;
-
   /**
    * @return The dest rect we'll use when drawing this image, in app units.
    *         Not necessarily contained in this item's bounds.
    */
-  nsRect GetDestRect() const final;
+  nsRect GetDestRect() const;
 
-  void UpdateDrawResult(mozilla::image::ImgDrawResult aResult) final {
-    nsDisplayItemGenericImageGeometry::UpdateDrawResult(this, aResult);
-  }
-
-  LayerState GetLayerState(nsDisplayListBuilder*, LayerManager*,
-                           const ContainerLayerParameters&) final;
   nsRect GetBounds(bool* aSnap) const {
     *aSnap = true;
-
-    nsImageFrame* imageFrame = static_cast<nsImageFrame*>(mFrame);
-    return imageFrame->GetInnerArea() + ToReferenceFrame();
+    return Frame()->GetContentRectRelativeToSelf() + ToReferenceFrame();
   }
 
   nsRect GetBounds(nsDisplayListBuilder*, bool* aSnap) const final {
@@ -490,8 +453,6 @@ class nsDisplayImage final : public nsDisplayImageContainer {
 
   nsRegion GetOpaqueRegion(nsDisplayListBuilder*, bool* aSnap) const final;
 
-  already_AddRefed<Layer> BuildLayer(nsDisplayListBuilder*, LayerManager*,
-                                     const ContainerLayerParameters&) final;
   bool CreateWebRenderCommands(mozilla::wr::DisplayListBuilder&,
                                mozilla::wr::IpcResourceUpdateQueue&,
                                const StackingContextHelper&,
@@ -504,4 +465,6 @@ class nsDisplayImage final : public nsDisplayImageContainer {
   nsCOMPtr<imgIContainer> mPrevImage;
 };
 
+}  // namespace mozilla
+
 #endif /* nsImageFrame_h___ */
