@@ -11,9 +11,10 @@
 #include "nsStyleTransformMatrix.h"
 #include "nsLayoutUtils.h"
 #include "nsPresContext.h"
-#include "nsSVGUtils.h"
+#include "mozilla/MotionPathUtils.h"
 #include "mozilla/ServoBindings.h"
 #include "mozilla/StyleAnimationValue.h"
+#include "mozilla/SVGUtils.h"
 #include "gfxMatrix.h"
 #include "gfxQuaternion.h"
 
@@ -35,7 +36,7 @@ namespace nsStyleTransformMatrix {
 // correctly, since when the frames are initially being reflowed, their
 // continuations all compute their bounding rects independently of each other
 // and consequently get the wrong value.
-//#define UNIFIED_CONTINUATIONS
+// #define UNIFIED_CONTINUATIONS
 
 void TransformReferenceBox::EnsureDimensionsAreCached() {
   if (mIsCached) {
@@ -46,15 +47,8 @@ void TransformReferenceBox::EnsureDimensionsAreCached() {
 
   mIsCached = true;
 
-  if (mFrame->GetStateBits() & NS_FRAME_SVG_LAYOUT) {
-    if (!nsLayoutUtils::SVGTransformBoxEnabled()) {
-      mX = -mFrame->GetPosition().x;
-      mY = -mFrame->GetPosition().y;
-      Size contextSize = nsSVGUtils::GetContextSize(mFrame);
-      mWidth = nsPresContext::CSSPixelsToAppUnits(contextSize.width);
-      mHeight = nsPresContext::CSSPixelsToAppUnits(contextSize.height);
-    } else if (mFrame->StyleDisplay()->mTransformBox ==
-               StyleGeometryBox::FillBox) {
+  if (mFrame->HasAnyStateBits(NS_FRAME_SVG_LAYOUT)) {
+    if (mFrame->StyleDisplay()->mTransformBox == StyleGeometryBox::FillBox) {
       // Percentages in transforms resolve against the SVG bbox, and the
       // transform is relative to the top-left of the SVG bbox.
       nsRect bboxInAppUnits = nsLayoutUtils::ComputeGeometryBox(
@@ -79,7 +73,7 @@ void TransformReferenceBox::EnsureDimensionsAreCached() {
       // transform is relative to {0,0} in current user space.
       mX = -mFrame->GetPosition().x;
       mY = -mFrame->GetPosition().y;
-      Size contextSize = nsSVGUtils::GetContextSize(mFrame);
+      Size contextSize = SVGUtils::GetContextSize(mFrame);
       mWidth = nsPresContext::CSSPixelsToAppUnits(contextSize.width);
       mHeight = nsPresContext::CSSPixelsToAppUnits(contextSize.height);
     }
@@ -114,11 +108,11 @@ void TransformReferenceBox::EnsureDimensionsAreCached() {
   mHeight = rect.Height();
 }
 
-void TransformReferenceBox::Init(const nsSize& aDimensions) {
+void TransformReferenceBox::Init(const nsRect& aDimensions) {
   MOZ_ASSERT(!mFrame && !mIsCached);
 
-  mX = 0;
-  mY = 0;
+  mX = aDimensions.x;
+  mY = aDimensions.y;
   mWidth = aDimensions.width;
   mHeight = aDimensions.height;
   mIsCached = true;
@@ -380,9 +374,16 @@ static void ProcessRotate3D(Matrix4x4& aMatrix, float aX, float aY, float aZ,
   aMatrix = temp * aMatrix;
 }
 
-static void ProcessPerspective(Matrix4x4& aMatrix, const Length& aLength) {
-  float depth = aLength.ToCSSPixels();
-  ApplyPerspectiveToMatrix(aMatrix, depth);
+static void ProcessPerspective(
+    Matrix4x4& aMatrix,
+    const StyleGenericPerspectiveFunction<Length>& aPerspective) {
+  if (aPerspective.IsNone()) {
+    return;
+  }
+  float p = aPerspective.AsLength().ToCSSPixels();
+  if (!std::isinf(p)) {
+    aMatrix.Perspective(std::max(p, 1.0f));
+  }
 }
 
 static void MatrixForTransformFunction(Matrix4x4& aMatrix,
@@ -491,19 +492,15 @@ static void ProcessTranslate(Matrix4x4& aMatrix,
     case StyleTranslate::Tag::None:
       return;
     case StyleTranslate::Tag::Translate:
-      return ProcessTranslate(aMatrix, aTranslate.AsTranslate()._0,
-                              aTranslate.AsTranslate()._1, aRefBox);
-    case StyleTranslate::Tag::Translate3D:
-      return ProcessTranslate3D(aMatrix, aTranslate.AsTranslate3D()._0,
-                                aTranslate.AsTranslate3D()._1,
-                                aTranslate.AsTranslate3D()._2, aRefBox);
+      return ProcessTranslate3D(aMatrix, aTranslate.AsTranslate()._0,
+                                aTranslate.AsTranslate()._1,
+                                aTranslate.AsTranslate()._2, aRefBox);
     default:
       MOZ_ASSERT_UNREACHABLE("Huh?");
   }
 }
 
-static void ProcessRotate(Matrix4x4& aMatrix, const StyleRotate& aRotate,
-                          TransformReferenceBox& aRefBox) {
+static void ProcessRotate(Matrix4x4& aMatrix, const StyleRotate& aRotate) {
   switch (aRotate.tag) {
     case StyleRotate::Tag::None:
       return;
@@ -519,17 +516,13 @@ static void ProcessRotate(Matrix4x4& aMatrix, const StyleRotate& aRotate,
   }
 }
 
-static void ProcessScale(Matrix4x4& aMatrix, const StyleScale& aScale,
-                         TransformReferenceBox& aRefBox) {
+static void ProcessScale(Matrix4x4& aMatrix, const StyleScale& aScale) {
   switch (aScale.tag) {
     case StyleScale::Tag::None:
       return;
     case StyleScale::Tag::Scale:
       return ProcessScaleHelper(aMatrix, aScale.AsScale()._0,
-                                aScale.AsScale()._1, 1.0f);
-    case StyleScale::Tag::Scale3D:
-      return ProcessScaleHelper(aMatrix, aScale.AsScale3D()._0,
-                                aScale.AsScale3D()._1, aScale.AsScale3D()._2);
+                                aScale.AsScale()._1, aScale.AsScale()._2);
     default:
       MOZ_ASSERT_UNREACHABLE("Huh?");
   }
@@ -537,24 +530,31 @@ static void ProcessScale(Matrix4x4& aMatrix, const StyleScale& aScale,
 
 Matrix4x4 ReadTransforms(const StyleTranslate& aTranslate,
                          const StyleRotate& aRotate, const StyleScale& aScale,
-                         const Maybe<MotionPathData>& aMotion,
+                         const Maybe<ResolvedMotionPathData>& aMotion,
                          const StyleTransform& aTransform,
                          TransformReferenceBox& aRefBox,
                          float aAppUnitsPerMatrixUnit) {
   Matrix4x4 result;
 
   ProcessTranslate(result, aTranslate, aRefBox);
-  ProcessRotate(result, aRotate, aRefBox);
-  ProcessScale(result, aScale, aRefBox);
+  ProcessRotate(result, aRotate);
+  ProcessScale(result, aScale);
 
   if (aMotion.isSome()) {
     // Create the equivalent translate and rotate function, according to the
     // order in spec. We combine the translate and then the rotate.
     // https://drafts.fxtf.org/motion-1/#calculating-path-transform
-    result.PreTranslate(aMotion->mTranslate.x, aMotion->mTranslate.y, 0.0);
+    //
+    // Besides, we have to shift the object by the delta between anchor-point
+    // and transform-origin, to make sure we rotate the object according to
+    // anchor-point.
+    result.PreTranslate(aMotion->mTranslate.x + aMotion->mShift.x,
+                        aMotion->mTranslate.y + aMotion->mShift.y, 0.0);
     if (aMotion->mRotate != 0.0) {
       result.RotateZ(aMotion->mRotate);
     }
+    // Shift the origin back to transform-origin.
+    result.PreTranslate(-aMotion->mShift.x, -aMotion->mShift.y, 0.0);
   }
 
   for (const StyleTransformOperation& op : aTransform.Operations()) {
@@ -568,6 +568,15 @@ Matrix4x4 ReadTransforms(const StyleTranslate& aTranslate,
   return result;
 }
 
+mozilla::CSSPoint Convert2DPosition(const mozilla::LengthPercentage& aX,
+                                    const mozilla::LengthPercentage& aY,
+                                    const CSSSize& aSize) {
+  return {
+      aX.ResolveToCSSPixels(aSize.width),
+      aY.ResolveToCSSPixels(aSize.height),
+  };
+}
+
 CSSPoint Convert2DPosition(const LengthPercentage& aX,
                            const LengthPercentage& aY,
                            TransformReferenceBox& aRefBox) {
